classdef (Abstract) finiteElements < handle
    % finiteElements
    % Abstract class to build Finite Elements    
    % It defines only universal error Objects.
    
    properties(Constant)
        wrongNumberInputs = MException('FINITEELEMENTS:WRONGNUMBERINPUTS',...
                        'The number of input arguments is wrong.');
        wrongNumberOutputs = MException('FINITEELEMENTS:WRONGNUMBERINPUTS',...
                        'The number of output arguments is wrong.');         
        wrongFormat = MException('FINITEELEMENTS:WRONGFORMAT',...
                                    'reading geometry data failed, maybe incorrect format.');
        wrongClass = MException('FINITEELEMENTS:WRONGCLASS',...
                                    'Wrong argument class.');
        wrongSize = MException('FINITEELEMENTS:WRONGSIZE',...
                                    'Wrong sized arguments.');                        
    end  
  
    properties(Abstract,Constant)
        idx
    end
    
    methods(Access = public)   
        % Assembling of all Matrices valid for all ...        
        function [varargout] = assema(obj,gridObj,cf,af,ff)
        % [K,M,F] = fem.assema(grid,c,a,f)
        % [S,F] = fem.assema(grid,c,a,f)
        %  The assemble method for finite elements.
        %  It assembles the Stiffness matrix K, the Mass Matrix M and the RHS 
        %  vector F in sparse format. There is a two argument call where S = K+M.  
        % (c) 2013 by Uwe Prüfert
            % specialized methods dep. on element type
            scl = superclasses(gridObj);
            for k = 1:length(scl)
                isgrid = strcmp(scl{k},'gridd');
                if isgrid
                    break;
                end
            end
            if ~isgrid
                obj.wrongClass.throw;
            end
            [K,M,F] = obj.createMatrixEntries(gridObj,cf,af,ff);
            [idx0,idx1,idx2] =  obj.makeIndex(gridObj.t(obj.idx,:),gridObj.nElements); 
            % do the sparse magic...  
            switch nargout
                case 2  
                    varargout{1} = sparse(idx1,idx2,K+M,gridObj.nPoints,gridObj.nPoints); 
                    varargout{2} = sparse(idx0,1,F,gridObj.nPoints,1);
                case 3  
                    varargout{1} = sparse(idx1,idx2,K,gridObj.nPoints,gridObj.nPoints);  
                    varargout{2} = sparse(idx1,idx2,M,gridObj.nPoints,gridObj.nPoints);
                    varargout{3} = sparse(idx0,1,F,gridObj.nPoints,1);
                otherwise
                    throw(obj.wrongNumberOutputs)
            end
        end
        
        function B = convection(obj,gridObj,b)
        % convection method of class finiteElements.
        % Universal convection assembling
        % B = fe.convection(grid,bvec) 
        % (C) 2013 by Uwe Prüfert.            
%             idx = gridObj.t(obj.idx,:);   
            % makeIndex method call, this method is in this class abstract! 
            scl = superclasses(gridObj);
            for k = 1:length(scl)
                isgrid = strcmp(scl{k},'gridd');
                if isgrid
                    break;
                end
            end
            if ~isgrid
                obj.wrongClass.throwAsCaller;
            end          
            [~,idx1,idx2] = obj.makeIndex(gridObj.t(obj.idx,:),gridObj.nElements);
            val = obj.createConvectionEntries(gridObj,b);
            B = sparse(idx1,idx2,val,gridObj.nPoints,gridObj.nPoints);    
        end
        
        
        function B = sparsityPattern(obj,gridObj,varargin)
            % sparsityPattern computes the sparsity pattern of the system
            % matrices
            % Usuage:
            % B = obj.sparsityPattern(gridObject)
            % B = obj.sparsityPattern(gridObject,A)
            % B = obj.sparsityPattern(gridObject,K,M,C,Q,H)
            % A can be every matrix, 
            % K,M,C,Q,H  should be the matrices from a call of obj.assema,
            % obj.assemb and obj.convection
            % The empty call computes all relevant matrices internally 
            
            switch length(varargin)
                case 0  % no matrices given
                    [K,M,~] = obj.assema(gridObj,'1','1','1');
                    [Q,~,H,~] = obj.assemb(gridObj);
                    C = obj.convection(gridObj,ones(size(gridObj.p(:,1))));
                    [indx1,indx2,s] = find(K+M+C+(H'*H)+Q);
                    B = sparse(indx1,indx2,s~=0);                    
                case 1  % whole matrix
                    [indx1,indx2,s] = find(varargin{1});
                    B = sparse(indx1,indx2,s~=0);                   
                case 5  % all relevant matrices in order K,M,C,Q,H
                    [indx1,indx2,s] = find(varargin{1}+...
                                           varargin{2}+...
                                           varargin{3}+...
                                           varargin{4}+...
                                           varargin{5}'*varargin{5});
                    B = sparse(indx1,indx2,s~=0); 
                otherwise
                    obj.wrongNumberInputs.throwAsCaller;
            end
        end
    end
    
    methods(Static,Access=public)
        function L = stiffSpring(M)
            % computes a guess for the stiff-spring coefficient
            % L = obj.stiffSpring(M)
            % M should be the Systems Matrix, e.g. M = K+M+C
            % (C) 2013 by Uwe Prüfert
            L = 1e3*norm(M,1);
        end
    end
    
    % declarations
    methods(Abstract,Access = public)   
        % Assembling of  Matrices abstract definition
        % to be specialized in subclasses
                  
        [K,M,F] = createMatrixEntries(obj,gridObj,cf,af,ff);
        val = createConvectionEntries(obj,gridObj,b);
        [idx0,idx1,idx2] = makeIndex(idx,nElements);
    end 
    
    methods(Abstract,Static,Access = public)   
        % Assembling of  Matrices abstract definition
        % to be specialized in subclasses
        [Q,G,H,R] = assemb(gridObj);                  
        
    end 
end

