%%
% BIFDETEC: Called from cont to detect bifurcation by checking for change of sign(det(A)), with A the Jacobian of the arclength 
% extended system. If yes, locate bif-point by bisection and save it. 
% If p.nc.neigdet=0 use LU-decomp to calc  sign(det(A)) 
% else use sign(det(A))=sign(product of neigdet EVals closest to zero)
% (assumption: negative Evals are all contained in muv) 
%
%  p=bifdetec(p,u1,tau1,Gu,Glam,ineg0)
%  p.u, p.sol.deta, ineg0    from previous continuation point data  
%  u1, lam1, tau1, Gu, Glam  at new point.
%
% Important settings: p.nc.neigdet, p.nc.bisecmax, p.nc.dsminbis, p.sw.bifloc.
%
% See also cont, folddetec, lu, spcalc, nloop, nloopext, genamat, stanparam
function p=bifdetec(p,u1,tau1,Gu,Glam,ineg0)
deta0=p.sol.deta; ds=p.sol.ds; xi=0.5; % use balanced xi for det(A) 
amat=genamat(p,Gu,Glam,tau1,xi,xi); % at new point 
if p.nc.neigdet==0 
  [L,U,P,Q]=lu(amat); p.sol.deta=full(prod(sign(diag(U)))*det(P)*det(Q));
else
  if p.sw.eigsstart==1; vs=size(amat,1); p.sw.evopts.v0=ones(vs,1)/vs; end 
  [V,mu]=eigs(amat,p.nc.neigdet,p.nc.eigref,p.sw.evopts); muv=mu*ones(1,p.nc.neigdet)';
  p.sol.deta=prod(sign(real(muv))); 
end
if(p.sol.restart>0 || abs(p.sol.deta-deta0)<1.1) % restart, or no bif, do nothing 
    return; 
end 
% Bifurcation detected, possibly cross-check with ineg;
ineg1=-1; % only occurs for spcalcsw=0
if p.sw.spcalc>0 [ineg1,p.sol.muv]=spcalc(Gu,p); end % EVals at new point, 
% also use at bifpoint if no ineg-change between p.u and bifpoint!
if(p.sw.verb>0); fprintf('   - bif between %g and %g\n',getlam(p),getlam(p,u1));  end 
% Now localize! 
u0=p.u; tau0=p.tau; ds=ds/2; bisecc=0; % start bisection  
while (bisecc<p.nc.bisecmax && abs(ds)>p.nc.dsminbis)  
   au0=u2au(p,u0,1); au1=u2au(p,u1,1); % get active variables
   if p.sw.bifloc==0 % tangent predictor 
      aun=au0+ds*tau0; %select entries of tau0 into full u = [upde;par]
   else if p.sw.bifloc==1 % secant predictor 
          aun=au0+0.5*(au1-au0); 
        else % quadratic predictor
          aun=0.25*(3*au0+au1)+0.5*ds*tau0; 
       end
   end
   un=au2u(p,aun,1); u0=au2u(p,au0,1); % merge with auxiliary variables
   if(p.sw.verb>1) fprintf('   - checking lam=%g ...',aun(p.nu+p.nc.nq+1)); end 
   if(p.sw.para==0 || (p.sw.para==1 && abs(tau0(p.nu+p.nc.nq+1))>p.nc.lamdtol)) % fixed lam corrector
      [un,res,iter,Gu,Glam]=nloop(p,un); 
   else
      [un,res,iter,Gu,Glam]=nloopext(p,un,ds,u0); % arclength corrector
   end
   if(res<2*p.nc.tol) % If step was (rather) OK, then 
     amat=genamat(p,Gu,Glam,tau0,p.sol.xi,p.sol.xiq);
%     amat=[[Gu Glam]; [p.sol.xi*tau0(1:p.nu+p.nc.nq)' (1-p.sol.xi)*tau0(p.nu+p.nc.nq+1)]];
     tau1=amat\[zeros(p.nu+p.nc.nq,1);1]; tau1=tau1/xinorm(tau1,p.sol.xi,p.nc.nq,p.sol.xiq); % tau at new point
     amat=genamat(p,Gu,Glam,tau1,xi,xi); % at new point 
     if p.nc.neigdet==0 
        [L,U,P,Q]=lu(amat); detab=full(prod(sign(diag(U)))*det(P)*det(Q));
     else
     if p.sw.eigsstart==1; vs=size(amat,1); p.sw.evopts.v0=ones(vs,1)/vs; end
       [V,mu]=eigs(amat,p.nc.neigdet,p.nc.eigref,p.sw.evopts); 
       muv=mu*ones(1,p.nc.neigdet)'; detab=sign(prod(real(muv))); 
     end
     if(p.sw.verb>1) fprintf('   - ok, sign(det(A))=%i\n',detab); end 
     tau0=tau1; % set tau, u0,u1 for the next step in bisec 
     if detab==deta0; % u_n still before bif, reset u0, keep u1; 
         ds=ds/2; 
     else  % u_n is behind bif., reset u1 to u0, i.e., flip direction:
        u1=u0; deta0=detab; ds=-ds/2; 
     end
     u0=un; 
   else
     if(p.sw.verb>1) fprintf('No convergence, loc. might be poor ...\n'); end
     ds=3*ds/4; % might have hit sing point, try a smaller step in predictor
   end % if res<2*p.nc.tol 
   bisecc=bisecc+1;
end % localization done 
%------------------------------------------------------- postprocessing!
us=p.u; taus=p.tau; tys=p.sol.ptype; % remember values behind bif 
p.u=u0; p.tau=tau0; % and store current values for save 
if p.sw.spcalc>0 [ineg,p.sol.muv]=spcalc(Gu,p);  % ineg at bifpoint
    if ineg-ineg0==0 p.sol.ineg=ineg1; end % no ineg-change between p.u and bifpoint, 
end
% use ineg at new point to indicate sign change at bifpoint
p.sol.ptype=1; % bifurcation point
if(p.sw.errcheck>0); p.sol.err=errcheck(p); end 
brout=[bradat(p); p.fuha.outfu(p,u0)]; % userfu to append to bif-branches  
brplot=brout(length(bradat(p))+p.plot.bpcmp);
p.branch=[p.branch brout]; p.fuha.savefu(p);   % put on branch and save
fname=[p.file.bpname,sprintf('%i',p.file.bcount),'.mat'];%save(fname,'p');    
fprintf('%4i  %7.5e (BP, saved to %s) bisection steps %i, last ds %g\n',...
    p.file.count,getlam(p),fname,bisecc,ds);
figure(p.plot.brfig); plot(getlam(p),brplot,'o');
if((p.file.count>0) && (mod(p.file.count,p.file.smod)==0)) % save to file 
   fname=[p.file.pname,sprintf('%i',p.file.count),'.mat']; % save(fname,'p'); 
   p.fuha.savefu(p,0); fprintf('   - also saved to %s\n',fname);
end 
p.u=us; p.tau=taus; p.sol.ptype=tys; p.sol.ineg=ineg1; % restore values behind bif 
p.file.bcount=p.file.bcount+1; p.file.count=p.file.count+1; 


